@@ -1,46 +1,64 @@
-use std::{
-    convert::TryFrom, future::Future, io, pin::Pin, sync::Arc, task::Context, task::Poll,
-};
+use std::{convert::TryFrom, future::Future, io, pin::Pin, task::Context, task::Poll};
 
-pub use tokio_rustls::{client::TlsStream, rustls::ClientConfig};
+pub use ntex_tls::rustls::TlsFilter;
+pub use tls_rustls::{ClientConfig, ServerName};
 
-use rust_tls::ServerName;
-use tokio_rustls::{self, TlsConnector};
+use ntex_tls::rustls::TlsConnector;
 
-use crate::rt::net::TcpStream;
+use crate::io::{utils::Boxed, Base, Io};
 use crate::service::{Service, ServiceFactory};
-use crate::util::Ready;
+use crate::util::{PoolId, Ready};
 
-use super::{Address, Connect, ConnectError, Connector};
+use super::{Address, Connect, ConnectError, Connector as BaseConnector};
 
 /// Rustls connector factory
-pub struct RustlsConnector<T> {
-    connector: Connector<T>,
-    config: Arc<ClientConfig>,
+pub struct Connector<T> {
+    connector: BaseConnector<T>,
+    inner: TlsConnector,
 }
 
-impl<T> RustlsConnector<T> {
-    pub fn new(config: Arc<ClientConfig>) -> Self {
-        RustlsConnector {
-            config,
-            connector: Connector::default(),
+impl<T> From<std::sync::Arc<ClientConfig>> for Connector<T> {
+    fn from(cfg: std::sync::Arc<ClientConfig>) -> Self {
+        Connector {
+            inner: TlsConnector::new(cfg),
+            connector: BaseConnector::default(),
         }
     }
 }
 
-impl<T: Address + 'static> RustlsConnector<T> {
+impl<T> Connector<T> {
+    pub fn new(config: ClientConfig) -> Self {
+        Connector {
+            inner: TlsConnector::new(std::sync::Arc::new(config)),
+            connector: BaseConnector::default(),
+        }
+    }
+
+    /// Set memory pool.
+    ///
+    /// Use specified memory pool for memory allocations. By default P0
+    /// memory pool is used.
+    pub fn memory_pool(self, id: PoolId) -> Self {
+        Self {
+            connector: self.connector.memory_pool(id),
+            inner: self.inner,
+        }
+    }
+}
+
+impl<T: Address + 'static> Connector<T> {
     /// Resolve and connect to remote host
     pub fn connect<U>(
         &self,
         message: U,
-    ) -> impl Future<Output = Result<TlsStream<TcpStream>, ConnectError>>
+    ) -> impl Future<Output = Result<Io<TlsFilter<Base>>, ConnectError>>
     where
         Connect<T>: From<U>,
     {
         let req = Connect::from(message);
         let host = req.host().split(':').next().unwrap().to_owned();
         let conn = self.connector.call(req);
-        let config = self.config.clone();
+        let connector = self.inner.clone();
 
         async move {
             let io = conn.await?;
@@ -48,47 +66,51 @@ impl<T: Address + 'static> RustlsConnector<T> {
 
             let host = ServerName::try_from(host.as_str())
                 .map_err(|e| io::Error::new(io::ErrorKind::Other, format!("{}", e)))?;
+            let connector = connector.server_name(host.clone());
 
-            match TlsConnector::from(config).connect(host.clone(), io).await {
+            match io.add_filter(connector).await {
                 Ok(io) => {
-                    trace!("SSL Handshake success: {:?}", &host);
+                    trace!("TLS Handshake success: {:?}", &host);
                     Ok(io)
                 }
                 Err(e) => {
-                    trace!("SSL Handshake error: {:?}", e);
+                    trace!("TLS Handshake error: {:?}", e);
                     Err(io::Error::new(io::ErrorKind::Other, format!("{}", e)).into())
                 }
             }
         }
     }
+
+    /// Produce sealed io stream (IoBoxed)
+    pub fn seal(self) -> Boxed<Connector<T>, Connect<T>> {
+        Boxed::new(self)
+    }
 }
 
-impl<T> Clone for RustlsConnector<T> {
+impl<T> Clone for Connector<T> {
     fn clone(&self) -> Self {
         Self {
-            config: self.config.clone(),
+            inner: self.inner.clone(),
             connector: self.connector.clone(),
         }
     }
 }
 
-impl<T: Address + 'static> ServiceFactory for RustlsConnector<T> {
-    type Request = Connect<T>;
-    type Response = TlsStream<TcpStream>;
+impl<T: Address, C> ServiceFactory<Connect<T>, C> for Connector<T> {
+    type Response = Io<TlsFilter<Base>>;
     type Error = ConnectError;
-    type Config = ();
-    type Service = RustlsConnector<T>;
+    type Service = Connector<T>;
     type InitError = ();
     type Future = Ready<Self::Service, Self::InitError>;
 
-    fn new_service(&self, _: ()) -> Self::Future {
+    #[inline]
+    fn new_service(&self, _: C) -> Self::Future {
         Ready::Ok(self.clone())
     }
 }
 
-impl<T: Address + 'static> Service for RustlsConnector<T> {
-    type Request = Connect<T>;
-    type Response = TlsStream<TcpStream>;
+impl<T: Address> Service<Connect<T>> for Connector<T> {
+    type Response = Io<TlsFilter<Base>>;
     type Error = ConnectError;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;
 
@@ -104,9 +126,10 @@ impl<T: Address + 'static> Service for RustlsConnector<T> {
 
 #[cfg(test)]
 mod tests {
+    use tls_rustls::{OwnedTrustAnchor, RootCertStore};
+
     use super::*;
     use crate::service::{Service, ServiceFactory};
-    use rust_tls::{OwnedTrustAnchor, RootCertStore};
 
     #[crate::rt_test]
     async fn test_rustls_connect() {
@@ -115,20 +138,20 @@ mod tests {
         });
 
         let mut cert_store = RootCertStore::empty();
-        cert_store.add_server_trust_anchors(
-            webpki_roots::TLS_SERVER_ROOTS.0.iter().map(|ta| {
+        cert_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(
+            |ta| {
                 OwnedTrustAnchor::from_subject_spki_name_constraints(
                     ta.subject,
                     ta.spki,
                     ta.name_constraints,
                 )
-            }),
-        );
+            },
+        ));
         let config = ClientConfig::builder()
             .with_safe_defaults()
             .with_root_certificates(cert_store)
             .with_no_client_auth();
-        let factory = RustlsConnector::new(Arc::new(config)).clone();
+        let factory = Connector::new(config).clone();
 
         let srv = factory.new_service(()).await.unwrap();
         let result = srv
