@@ -1,14 +1,16 @@
 use std::task::{Context, Poll};
 use std::{collections::VecDeque, future::Future, io, net::SocketAddr, pin::Pin};
 
-use crate::rt::net::TcpStream;
+use crate::io::{types, utils::Boxed, Io};
+use crate::rt::tcp_connect_in;
 use crate::service::{Service, ServiceFactory};
-use crate::util::{Either, Ready};
+use crate::util::{Either, PoolId, PoolRef, Ready};
 
 use super::{Address, Connect, ConnectError, Resolver};
 
 pub struct Connector<T> {
     resolver: Resolver<T>,
+    pool: PoolRef,
 }
 
 impl<T> Connector<T> {
@@ -16,28 +18,41 @@ impl<T> Connector<T> {
     pub fn new() -> Self {
         Connector {
             resolver: Resolver::new(),
+            pool: PoolId::P0.pool_ref(),
         }
     }
+
+    /// Set memory pool.
+    ///
+    /// Use specified memory pool for memory allocations. By default P0
+    /// memory pool is used.
+    pub fn memory_pool(mut self, id: PoolId) -> Self {
+        self.pool = id.pool_ref();
+        self
+    }
 }
 
 impl<T: Address> Connector<T> {
     /// Resolve and connect to remote host
-    pub fn connect<U>(
-        &self,
-        message: U,
-    ) -> impl Future<Output = Result<TcpStream, ConnectError>>
+    pub fn connect<U>(&self, message: U) -> impl Future<Output = Result<Io, ConnectError>>
     where
         Connect<T>: From<U>,
     {
-        ConnectServiceResponse::new(self.resolver.call(message.into()))
+        ConnectServiceResponse {
+            state: ConnectState::Resolve(self.resolver.call(message.into())),
+            pool: self.pool,
+        }
+    }
+
+    /// Produce sealed io stream (IoBoxed)
+    pub fn seal(self) -> Boxed<Connector<T>, Connect<T>> {
+        Boxed::new(self)
     }
 }
 
 impl<T> Default for Connector<T> {
     fn default() -> Self {
-        Connector {
-            resolver: Resolver::default(),
-        }
+        Connector::new()
     }
 }
 
@@ -45,28 +60,26 @@ impl<T> Clone for Connector<T> {
     fn clone(&self) -> Self {
         Connector {
             resolver: self.resolver.clone(),
+            pool: self.pool,
         }
     }
 }
 
-impl<T: Address> ServiceFactory for Connector<T> {
-    type Request = Connect<T>;
-    type Response = TcpStream;
+impl<T: Address, C> ServiceFactory<Connect<T>, C> for Connector<T> {
+    type Response = Io;
     type Error = ConnectError;
-    type Config = ();
     type Service = Connector<T>;
     type InitError = ();
     type Future = Ready<Self::Service, Self::InitError>;
 
     #[inline]
-    fn new_service(&self, _: ()) -> Self::Future {
+    fn new_service(&self, _: C) -> Self::Future {
         Ready::Ok(self.clone())
     }
 }
 
-impl<T: Address> Service for Connector<T> {
-    type Request = Connect<T>;
-    type Response = TcpStream;
+impl<T: Address> Service<Connect<T>> for Connector<T> {
+    type Response = Io;
     type Error = ConnectError;
     type Future = ConnectServiceResponse<T>;
 
@@ -82,25 +95,27 @@ impl<T: Address> Service for Connector<T> {
 }
 
 enum ConnectState<T: Address> {
-    Resolve(<Resolver<T> as Service>::Future),
+    Resolve(<Resolver<T> as Service<Connect<T>>>::Future),
     Connect(TcpConnectorResponse<T>),
 }
 
 #[doc(hidden)]
 pub struct ConnectServiceResponse<T: Address> {
     state: ConnectState<T>,
+    pool: PoolRef,
 }
 
 impl<T: Address> ConnectServiceResponse<T> {
-    pub(super) fn new(fut: <Resolver<T> as Service>::Future) -> Self {
-        ConnectServiceResponse {
+    pub(super) fn new(fut: <Resolver<T> as Service<Connect<T>>>::Future) -> Self {
+        Self {
             state: ConnectState::Resolve(fut),
+            pool: PoolId::P0.pool_ref(),
         }
     }
 }
 
 impl<T: Address> Future for ConnectServiceResponse<T> {
-    type Output = Result<TcpStream, ConnectError>;
+    type Output = Result<Io, ConnectError>;
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         match self.state {
@@ -112,7 +127,7 @@ impl<T: Address> Future for ConnectServiceResponse<T> {
 
                     if let Some(addr) = addr {
                         self.state = ConnectState::Connect(TcpConnectorResponse::new(
-                            req, port, addr,
+                            req, port, addr, self.pool,
                         ));
                         self.poll(cx)
                     } else if let Some(addr) = req.addr() {
@@ -120,6 +135,7 @@ impl<T: Address> Future for ConnectServiceResponse<T> {
                             req,
                             addr.port(),
                             Either::Left(addr),
+                            self.pool,
                         ));
                         self.poll(cx)
                     } else {
@@ -138,7 +154,8 @@ struct TcpConnectorResponse<T> {
     req: Option<T>,
     port: u16,
     addrs: Option<VecDeque<SocketAddr>>,
-    stream: Option<Pin<Box<dyn Future<Output = Result<TcpStream, io::Error>>>>>,
+    stream: Option<Pin<Box<dyn Future<Output = Result<Io, io::Error>>>>>,
+    pool: PoolRef,
 }
 
 impl<T: Address> TcpConnectorResponse<T> {
@@ -146,6 +163,7 @@ impl<T: Address> TcpConnectorResponse<T> {
         req: T,
         port: u16,
         addr: Either<SocketAddr, VecDeque<SocketAddr>>,
+        pool: PoolRef,
     ) -> TcpConnectorResponse<T> {
         trace!(
             "TCP connector - connecting to {:?} port:{}",
@@ -156,13 +174,15 @@ impl<T: Address> TcpConnectorResponse<T> {
         match addr {
             Either::Left(addr) => TcpConnectorResponse {
                 req: Some(req),
-                port,
                 addrs: None,
-                stream: Some(Box::pin(TcpStream::connect(addr))),
+                stream: Some(Box::pin(tcp_connect_in(addr, pool))),
+                pool,
+                port,
             },
             Either::Right(addrs) => TcpConnectorResponse {
-                req: Some(req),
                 port,
+                pool,
+                req: Some(req),
                 addrs: Some(addrs),
                 stream: None,
             },
@@ -181,7 +201,7 @@ impl<T: Address> TcpConnectorResponse<T> {
 }
 
 impl<T: Address> Future for TcpConnectorResponse<T> {
-    type Output = Result<TcpStream, ConnectError>;
+    type Output = Result<Io, ConnectError>;
 
     fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         let this = self.get_mut();
@@ -191,16 +211,11 @@ impl<T: Address> Future for TcpConnectorResponse<T> {
             if let Some(new) = this.stream.as_mut() {
                 match new.as_mut().poll(cx) {
                     Poll::Ready(Ok(sock)) => {
-                        if let Err(err) = sock.set_nodelay(true) {
-                            if !this.can_continue(&err) {
-                                return Poll::Ready(Err(err.into()));
-                            }
-                        }
-
                         let req = this.req.take().unwrap();
                         trace!(
                             "TCP connector - successfully connected to connecting to {:?} - {:?}",
-                            req.host(), sock.peer_addr()
+                            req.host(),
+                            sock.query::<types::PeerAddr>().get()
                         );
                         return Poll::Ready(Ok(sock));
                     }
@@ -215,7 +230,7 @@ impl<T: Address> Future for TcpConnectorResponse<T> {
 
             // try to connect
             let addr = this.addrs.as_mut().unwrap().pop_front().unwrap();
-            this.stream = Some(Box::pin(TcpStream::connect(addr)));
+            this.stream = Some(Box::pin(tcp_connect_in(addr, this.pool)));
         }
     }
 }
