@@ -2,8 +2,6 @@ use std::cell::{Ref, RefMut};
 use std::task::{Context, Poll};
 use std::{fmt, future::Future, marker::PhantomData, mem, pin::Pin};
 
-use bytes::{Bytes, BytesMut};
-use futures_core::Stream;
 use serde::de::DeserializeOwned;
 
 #[cfg(feature = "cookie")]
@@ -13,7 +11,8 @@ use crate::http::error::PayloadError;
 use crate::http::header::{AsName, HeaderValue, CONTENT_LENGTH};
 use crate::http::{HeaderMap, StatusCode, Version};
 use crate::http::{HttpMessage, Payload, ResponseHead};
-use crate::util::Extensions;
+use crate::util::{Bytes, BytesMut, Extensions};
+use crate::Stream;
 
 use super::error::JsonPayloadError;
 
@@ -46,8 +45,8 @@ impl HttpMessage for ClientResponse {
         if self.message_extensions().get::<Cookies>().is_none() {
             let mut cookies = Vec::new();
             for hdr in self.message_headers().get_all(&SET_COOKIE) {
-                let s = std::str::from_utf8(hdr.as_bytes())
-                    .map_err(CookieParseError::from)?;
+                let s =
+                    std::str::from_utf8(hdr.as_bytes()).map_err(CookieParseError::from)?;
                 cookies.push(Cookie::parse_encoded(s)?.into_owned());
             }
             self.message_extensions_mut().insert(Cookies(cookies));
@@ -64,6 +63,10 @@ impl ClientResponse {
         ClientResponse { head, payload }
     }
 
+    pub(crate) fn with_empty_payload(head: ResponseHead) -> Self {
+        ClientResponse::new(head, Payload::None)
+    }
+
     #[inline]
     pub(crate) fn head(&self) -> &ResponseHead {
         &self.head
@@ -137,10 +140,7 @@ impl ClientResponse {
 impl Stream for ClientResponse {
     type Item = Result<Bytes, PayloadError>;
 
-    fn poll_next(
-        self: Pin<&mut Self>,
-        cx: &mut Context<'_>,
-    ) -> Poll<Option<Self::Item>> {
+    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
         Pin::new(&mut self.get_mut().payload).poll_next(cx)
     }
 }
@@ -298,9 +298,7 @@ where
 
         if let Some(len) = self.length.take() {
             if len > self.fut.as_ref().unwrap().limit {
-                return Poll::Ready(Err(JsonPayloadError::Payload(
-                    PayloadError::Overflow,
-                )));
+                return Poll::Ready(Err(JsonPayloadError::Payload(PayloadError::Overflow)));
             }
         }
 
@@ -367,8 +365,7 @@ mod tests {
             _ => unreachable!("error"),
         }
 
-        let mut req =
-            TestResponse::with_header(header::CONTENT_LENGTH, "1000000").finish();
+        let mut req = TestResponse::with_header(header::CONTENT_LENGTH, "1000000").finish();
         match req.body().await.err().unwrap() {
             PayloadError::Overflow => (),
             _ => unreachable!("error"),
