@@ -5,12 +5,12 @@ use crate::service::{IntoService, Service};
 use crate::{util::poll_fn, Sink, Stream};
 
 pin_project_lite::pin_project! {
-    pub struct Dispatcher<R, S, T, U>
+    pub struct Dispatcher<Req, R, S, T, U>
     where
         R: 'static,
-        S: Service<Response = Option<R>>,
+        S: Service<Req, Response = Option<R>>,
         S: 'static,
-        T: Stream<Item = Result<S::Request, S::Error>>,
+        T: Stream<Item = Result<Req, S::Error>>,
         T: Unpin,
         U: Sink<Result<R, S::Error>>,
         U: Unpin,
@@ -24,18 +24,18 @@ pin_project_lite::pin_project! {
     }
 }
 
-impl<R, S, T, U> Dispatcher<R, S, T, U>
+impl<Req, R, S, T, U> Dispatcher<Req, R, S, T, U>
 where
     R: 'static,
-    S: Service<Response = Option<R>> + 'static,
+    S: Service<Req, Response = Option<R>> + 'static,
     S::Error: fmt::Debug,
-    T: Stream<Item = Result<S::Request, S::Error>> + Unpin,
+    T: Stream<Item = Result<Req, S::Error>> + Unpin,
     U: Sink<Result<R, S::Error>> + Unpin + 'static,
     U::Error: fmt::Debug,
 {
     pub fn new<F>(stream: T, sink: U, service: F) -> Self
     where
-        F: IntoService<S>,
+        F: IntoService<S, Req>,
     {
         Dispatcher {
             stream,
@@ -47,12 +47,13 @@ where
     }
 }
 
-impl<R, S, T, U> Future for Dispatcher<R, S, T, U>
+impl<Req, R, S, T, U> Future for Dispatcher<Req, R, S, T, U>
 where
     R: 'static,
-    S: Service<Response = Option<R>> + 'static,
-    S::Error: fmt::Debug,
-    T: Stream<Item = Result<S::Request, S::Error>> + Unpin,
+    S: Service<Req, Response = Option<R>> + 'static,
+    S::Future: 'static,
+    S::Error: fmt::Debug + 'static,
+    T: Stream<Item = Result<Req, S::Error>> + Unpin,
     U: Sink<Result<R, S::Error>> + Unpin + 'static,
     U::Error: fmt::Debug,
 {
@@ -92,8 +93,7 @@ where
                     }
                 }
                 Poll::Ready(Ok(_)) => {
-                    if let Poll::Ready(Some(item)) = Pin::new(&mut this.rx).poll_next(cx)
-                    {
+                    if let Poll::Ready(Some(item)) = Pin::new(&mut this.rx).poll_next(cx) {
                         match item {
                             Ok(Some(item)) => {
                                 if let Err(e) = Pin::new(this.sink.as_mut().unwrap())
@@ -161,15 +161,11 @@ where
 
 #[cfg(test)]
 mod tests {
-    use bytes::BytesMut;
-    use bytestring::ByteString;
-    use std::{cell::Cell, rc::Rc, time::Duration};
+    use std::{cell::Cell, rc::Rc};
 
     use super::*;
-    use crate::channel::mpsc;
-    use crate::codec::Encoder;
-    use crate::rt::time::sleep;
-    use crate::{util::next, ws};
+    use crate::util::{next, ByteString, BytesMut};
+    use crate::{channel::mpsc, codec::Encoder, time::sleep, time::Millis, ws};
 
     #[crate::rt_test]
     async fn test_basic() {
@@ -184,7 +180,7 @@ mod tests {
         let disp = Dispatcher::new(
             decoder,
             encoder,
-            crate::fn_service(move |_| {
+            crate::service::fn_service(move |_| {
                 counter2.set(counter2.get() + 1);
                 async { Ok(Some(ws::Message::Text(ByteString::from_static("test")))) }
             }),
@@ -204,7 +200,7 @@ mod tests {
         assert_eq!(data, b"\x81\x04test".as_ref());
 
         drop(tx);
-        sleep(Duration::from_millis(10)).await;
+        sleep(Millis(10)).await;
         assert!(next(&mut rx).await.is_none());
 
         assert_eq!(counter.get(), 1);
