@@ -1,7 +1,6 @@
 //! Various helpers for ntex applications to use during testing.
 use std::{
-    convert::TryFrom, error::Error, fmt, net, net::SocketAddr, rc::Rc, sync::mpsc,
-    thread,
+    convert::TryFrom, error::Error, fmt, net, net::SocketAddr, rc::Rc, sync::mpsc, thread,
 };
 
 #[cfg(feature = "cookie")]
@@ -9,10 +8,8 @@ use coo_kie::Cookie;
 use serde::de::DeserializeOwned;
 use serde::Serialize;
 
-use crate::codec::{AsyncRead, AsyncWrite};
 use crate::http::body::MessageBody;
-use crate::http::client::error::WsClientError;
-use crate::http::client::{ws, Client, ClientRequest, ClientResponse, Connector};
+use crate::http::client::{Client, ClientRequest, ClientResponse, Connector};
 use crate::http::error::{HttpError, PayloadError, ResponseError};
 use crate::http::header::{HeaderName, HeaderValue, CONTENT_TYPE};
 use crate::http::test::TestRequest as HttpTestRequest;
@@ -23,7 +20,8 @@ use crate::service::{
 };
 use crate::time::{sleep, Millis, Seconds};
 use crate::util::{next, Bytes, BytesMut, Extensions, Ready};
-use crate::{rt::System, server::Server, Stream};
+use crate::ws::{error::WsClientError, WsClient, WsConnection};
+use crate::{io::Sealed, rt::System, server::Server, Stream};
 
 use crate::web::config::AppConfig;
 use crate::web::error::{DefaultError, ErrorRenderer};
@@ -32,22 +30,17 @@ use crate::web::rmap::ResourceMap;
 use crate::web::{FromRequest, HttpResponse, Responder, WebRequest, WebResponse};
 
 /// Create service that always responds with `HttpResponse::Ok()`
-pub fn ok_service<Err: ErrorRenderer>() -> impl Service<
-    Request = WebRequest<Err>,
-    Response = WebResponse,
-    Error = std::convert::Infallible,
-> {
+pub fn ok_service<Err: ErrorRenderer>(
+) -> impl Service<WebRequest<Err>, Response = WebResponse, Error = std::convert::Infallible>
+{
     default_service::<Err>(StatusCode::OK)
 }
 
 /// Create service that responds with response with specified status code
 pub fn default_service<Err: ErrorRenderer>(
     status_code: StatusCode,
-) -> impl Service<
-    Request = WebRequest<Err>,
-    Response = WebResponse,
-    Error = std::convert::Infallible,
-> {
+) -> impl Service<WebRequest<Err>, Response = WebResponse, Error = std::convert::Infallible>
+{
     (move |req: WebRequest<Err>| {
         Ready::Ok(req.into_response(HttpResponse::build(status_code).finish()))
     })
@@ -79,15 +72,10 @@ pub fn default_service<Err: ErrorRenderer>(
 /// ```
 pub async fn init_service<R, S, E>(
     app: R,
-) -> impl Service<Request = Request, Response = WebResponse, Error = E>
+) -> impl Service<Request, Response = WebResponse, Error = E>
 where
-    R: IntoServiceFactory<S>,
-    S: ServiceFactory<
-        Config = AppConfig,
-        Request = Request,
-        Response = WebResponse,
-        Error = E,
-    >,
+    R: IntoServiceFactory<S, Request, AppConfig>,
+    S: ServiceFactory<Request, AppConfig, Response = WebResponse, Error = E>,
     S::InitError: std::fmt::Debug,
 {
     let srv = app.into_factory();
@@ -119,7 +107,7 @@ where
 /// ```
 pub async fn call_service<S, R, E>(app: &S, req: R) -> S::Response
 where
-    S: Service<Request = R, Response = WebResponse, Error = E>,
+    S: Service<R, Response = WebResponse, Error = E>,
     E: std::fmt::Debug,
 {
     app.call(req).await.unwrap()
@@ -152,7 +140,7 @@ where
 /// ```
 pub async fn read_response<S>(app: &S, req: Request) -> Bytes
 where
-    S: Service<Request = Request, Response = WebResponse>,
+    S: Service<Request, Response = WebResponse>,
 {
     let mut resp = app
         .call(req)
@@ -251,7 +239,7 @@ where
 /// ```
 pub async fn read_response_json<S, T>(app: &S, req: Request) -> T
 where
-    S: Service<Request = Request, Response = WebResponse>,
+    S: Service<Request, Response = WebResponse>,
     T: DeserializeOwned,
 {
     let body = read_response::<S>(app, req).await;
@@ -465,15 +453,12 @@ impl TestRequest {
 
     /// Complete request creation and generate `Request` instance
     pub fn to_request(mut self) -> Request {
-        let mut req = self.req.finish();
-        req.head_mut().peer_addr = self.peer_addr;
-        req
+        self.req.finish()
     }
 
     /// Complete request creation and generate `WebRequest` instance
     pub fn to_srv_request(mut self) -> WebRequest<DefaultError> {
-        let (mut head, payload) = self.req.finish().into_parts();
-        head.peer_addr = self.peer_addr;
+        let (head, payload) = self.req.finish().into_parts();
         *self.path.get_mut() = head.uri.clone();
 
         WebRequest::new(HttpRequest::new(
@@ -494,8 +479,7 @@ impl TestRequest {
 
     /// Complete request creation and generate `HttpRequest` instance
     pub fn to_http_request(mut self) -> HttpRequest {
-        let (mut head, payload) = self.req.finish().into_parts();
-        head.peer_addr = self.peer_addr;
+        let (head, payload) = self.req.finish().into_parts();
         *self.path.get_mut() = head.uri.clone();
 
         HttpRequest::new(
@@ -511,8 +495,7 @@ impl TestRequest {
 
     /// Complete request creation and generate `HttpRequest` and `Payload` instances
     pub fn to_http_parts(mut self) -> (HttpRequest, Payload) {
-        let (mut head, payload) = self.req.finish().into_parts();
-        head.peer_addr = self.peer_addr;
+        let (head, payload) = self.req.finish().into_parts();
         *self.path.get_mut() = head.uri.clone();
 
         let req = HttpRequest::new(
@@ -558,12 +541,11 @@ impl TestRequest {
 pub fn server<F, I, S, B>(factory: F) -> TestServer
 where
     F: Fn() -> I + Send + Clone + 'static,
-    I: IntoServiceFactory<S>,
-    S: ServiceFactory<Config = AppConfig, Request = Request> + 'static,
-    S::Error: ResponseError + 'static,
+    I: IntoServiceFactory<S, Request, AppConfig>,
+    S: ServiceFactory<Request, AppConfig> + 'static,
+    S::Error: ResponseError,
     S::InitError: fmt::Debug,
-    S::Response: Into<HttpResponse<B>> + 'static,
-    <S::Service as Service>::Future: 'static,
+    S::Response: Into<HttpResponse<B>>,
     B: MessageBody + 'static,
 {
     server_with(TestServerConfig::default(), factory)
@@ -597,12 +579,11 @@ where
 pub fn server_with<F, I, S, B>(cfg: TestServerConfig, factory: F) -> TestServer
 where
     F: Fn() -> I + Send + Clone + 'static,
-    I: IntoServiceFactory<S>,
-    S: ServiceFactory<Config = AppConfig, Request = Request> + 'static,
-    S::Error: ResponseError + 'static,
+    I: IntoServiceFactory<S, Request, AppConfig>,
+    S: ServiceFactory<Request, AppConfig> + 'static,
+    S::Error: ResponseError,
     S::InitError: fmt::Debug,
-    S::Response: Into<HttpResponse<B>> + 'static,
-    <S::Service as Service>::Future: 'static,
+    S::Response: Into<HttpResponse<B>>,
     B: MessageBody + 'static,
 {
     let (tx, rx) = mpsc::channel();
@@ -617,7 +598,7 @@ where
 
     // run server in separate thread
     thread::spawn(move || {
-        let mut sys = System::new("ntex-test-server");
+        let sys = System::new("ntex-test-server");
 
         let cfg = cfg.clone();
         let factory = factory.clone();
@@ -625,39 +606,36 @@ where
         let tcp = net::TcpListener::bind("127.0.0.1:0").unwrap();
         let local_addr = tcp.local_addr().unwrap();
 
-        let srv = sys.exec(|| {
+        let srv = sys.exec(move || {
             let builder = Server::build().workers(1).disable_signals();
 
             match cfg.stream {
                 StreamType::Tcp => match cfg.tp {
-                    HttpVer::Http1 => builder.listen("test", tcp, move || {
+                    HttpVer::Http1 => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(false, local_addr, format!("{}", local_addr));
                         HttpService::build()
                             .client_timeout(ctimeout)
                             .h1(map_config(factory(), move |_| cfg.clone()))
-                            .tcp()
                     }),
-                    HttpVer::Http2 => builder.listen("test", tcp, move || {
+                    HttpVer::Http2 => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(false, local_addr, format!("{}", local_addr));
                         HttpService::build()
                             .client_timeout(ctimeout)
                             .h2(map_config(factory(), move |_| cfg.clone()))
-                            .tcp()
                     }),
-                    HttpVer::Both => builder.listen("test", tcp, move || {
+                    HttpVer::Both => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(false, local_addr, format!("{}", local_addr));
                         HttpService::build()
                             .client_timeout(ctimeout)
                             .finish(map_config(factory(), move |_| cfg.clone()))
-                            .tcp()
                     }),
                 },
                 #[cfg(feature = "openssl")]
                 StreamType::Openssl(acceptor) => match cfg.tp {
-                    HttpVer::Http1 => builder.listen("test", tcp, move || {
+                    HttpVer::Http1 => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(true, local_addr, format!("{}", local_addr));
                         HttpService::build()
@@ -665,7 +643,7 @@ where
                             .h1(map_config(factory(), move |_| cfg.clone()))
                             .openssl(acceptor.clone())
                     }),
-                    HttpVer::Http2 => builder.listen("test", tcp, move || {
+                    HttpVer::Http2 => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(true, local_addr, format!("{}", local_addr));
                         HttpService::build()
@@ -673,7 +651,7 @@ where
                             .h2(map_config(factory(), move |_| cfg.clone()))
                             .openssl(acceptor.clone())
                     }),
-                    HttpVer::Both => builder.listen("test", tcp, move || {
+                    HttpVer::Both => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(true, local_addr, format!("{}", local_addr));
                         HttpService::build()
@@ -684,7 +662,7 @@ where
                 },
                 #[cfg(feature = "rustls")]
                 StreamType::Rustls(config) => match cfg.tp {
-                    HttpVer::Http1 => builder.listen("test", tcp, move || {
+                    HttpVer::Http1 => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(true, local_addr, format!("{}", local_addr));
                         HttpService::build()
@@ -692,7 +670,7 @@ where
                             .h1(map_config(factory(), move |_| cfg.clone()))
                             .rustls(config.clone())
                     }),
-                    HttpVer::Http2 => builder.listen("test", tcp, move || {
+                    HttpVer::Http2 => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(true, local_addr, format!("{}", local_addr));
                         HttpService::build()
@@ -700,7 +678,7 @@ where
                             .h2(map_config(factory(), move |_| cfg.clone()))
                             .rustls(config.clone())
                     }),
-                    HttpVer::Both => builder.listen("test", tcp, move || {
+                    HttpVer::Both => builder.listen("test", tcp, move |_| {
                         let cfg =
                             AppConfig::new(true, local_addr, format!("{}", local_addr));
                         HttpService::build()
@@ -711,7 +689,7 @@ where
                 },
             }
             .unwrap()
-            .start()
+            .run()
         });
 
         tx.send((System::current(), srv, local_addr)).unwrap();
@@ -724,7 +702,7 @@ where
         let connector = {
             #[cfg(feature = "openssl")]
             {
-                use open_ssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
+                use tls_openssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
 
                 let mut builder = SslConnector::builder(SslMethod::tls()).unwrap();
                 builder.set_verify(SslVerifyMode::NONE);
@@ -735,7 +713,7 @@ where
                     .lifetime(Seconds::ZERO)
                     .keep_alive(Seconds(30))
                     .timeout(Millis(30_000))
-                    .disconnect_timeout(Millis(3_000))
+                    .disconnect_timeout(Millis(5_000))
                     .openssl(builder.build())
                     .finish()
             }
@@ -782,9 +760,9 @@ enum HttpVer {
 enum StreamType {
     Tcp,
     #[cfg(feature = "openssl")]
-    Openssl(open_ssl::ssl::SslAcceptor),
+    Openssl(tls_openssl::ssl::SslAcceptor),
     #[cfg(feature = "rustls")]
-    Rustls(rust_tls::ServerConfig),
+    Rustls(tls_rustls::ServerConfig),
 }
 
 impl fmt::Debug for StreamType {
@@ -834,14 +812,14 @@ impl TestServerConfig {
 
     /// Start openssl server
     #[cfg(feature = "openssl")]
-    pub fn openssl(mut self, acceptor: open_ssl::ssl::SslAcceptor) -> Self {
+    pub fn openssl(mut self, acceptor: tls_openssl::ssl::SslAcceptor) -> Self {
         self.stream = StreamType::Openssl(acceptor);
         self
     }
 
     /// Start rustls server
     #[cfg(feature = "rustls")]
-    pub fn rustls(mut self, config: rust_tls::ServerConfig) -> Self {
+    pub fn rustls(mut self, config: tls_rustls::ServerConfig) -> Self {
         self.stream = StreamType::Rustls(config);
         self
     }
@@ -928,19 +906,47 @@ impl TestServer {
     }
 
     /// Connect to websocket server at a given path
-    pub async fn ws_at(
-        &self,
-        path: &str,
-    ) -> Result<ws::WsConnection<impl AsyncRead + AsyncWrite>, WsClientError> {
-        let url = self.url(path);
-        let connect = self.client.ws(url).connect();
-        connect.await
+    pub async fn ws_at(&self, path: &str) -> Result<WsConnection<Sealed>, WsClientError> {
+        if self.ssl {
+            #[cfg(feature = "openssl")]
+            {
+                use tls_openssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
+
+                let mut builder = SslConnector::builder(SslMethod::tls()).unwrap();
+                builder.set_verify(SslVerifyMode::NONE);
+                let _ = builder
+                    .set_alpn_protos(b"\x08http/1.1")
+                    .map_err(|e| log::error!("Cannot set alpn protocol: {:?}", e));
+
+                WsClient::build(self.url(path))
+                    .address(self.addr)
+                    .timeout(Seconds(30))
+                    .openssl(builder.build())
+                    .take()
+                    .finish()
+                    .unwrap()
+                    .connect()
+                    .await
+                    .map(|ws| ws.seal())
+            }
+            #[cfg(not(feature = "openssl"))]
+            {
+                panic!("openssl feature is required")
+            }
+        } else {
+            WsClient::build(self.url(path))
+                .address(self.addr)
+                .timeout(Seconds(30))
+                .finish()
+                .unwrap()
+                .connect()
+                .await
+                .map(|ws| ws.seal())
+        }
     }
 
     /// Connect to a websocket server
-    pub async fn ws(
-        &self,
-    ) -> Result<ws::WsConnection<impl AsyncRead + AsyncWrite>, WsClientError> {
+    pub async fn ws(&self) -> Result<WsConnection<Sealed>, WsClientError> {
         self.ws_at("/").await
     }
 
@@ -979,10 +985,7 @@ mod tests {
             .to_http_request();
         assert!(req.headers().contains_key(header::CONTENT_TYPE));
         assert!(req.headers().contains_key(header::DATE));
-        assert_eq!(
-            req.head().peer_addr,
-            Some("127.0.0.1:8081".parse().unwrap())
-        );
+        // assert_eq!(req.peer_addr(), Some("127.0.0.1:8081".parse().unwrap()));
         assert_eq!(&req.match_info()["test"], "123");
         assert_eq!(req.version(), Version::HTTP_2);
         let data = req.app_data::<web::types::Data<u64>>().unwrap();
@@ -997,12 +1000,9 @@ mod tests {
             App::new().service(
                 web::resource("/index.html")
                     .route(web::put().to(|| async { HttpResponse::Ok().body("put!") }))
+                    .route(web::patch().to(|| async { HttpResponse::Ok().body("patch!") }))
                     .route(
-                        web::patch().to(|| async { HttpResponse::Ok().body("patch!") }),
-                    )
-                    .route(
-                        web::delete()
-                            .to(|| async { HttpResponse::Ok().body("delete!") }),
+                        web::delete().to(|| async { HttpResponse::Ok().body("delete!") }),
                     ),
             ),
         )
@@ -1032,9 +1032,11 @@ mod tests {
     #[crate::rt_test]
     async fn test_response() {
         let app =
-            init_service(App::new().service(web::resource("/index.html").route(
-                web::post().to(|| async { HttpResponse::Ok().body("welcome!") }),
-            )))
+            init_service(App::new().service(
+                web::resource("/index.html").route(
+                    web::post().to(|| async { HttpResponse::Ok().body("welcome!") }),
+                ),
+            ))
             .await;
 
         let req = TestRequest::post()
@@ -1205,32 +1207,6 @@ mod tests {
         assert_eq!(srv.load_body(res).await.unwrap(), Bytes::new());
     }
 
-    #[crate::rt_test]
-    async fn test_h2_tcp() {
-        let srv = server_with(TestServerConfig::default().h2(), || {
-            App::new().service(
-                web::resource("/").route(web::get().to(|| async { HttpResponse::Ok() })),
-            )
-        });
-
-        let client = Client::build()
-            .connector(
-                Connector::default()
-                    .secure_connector(Service::map(
-                        crate::connect::Connector::default(),
-                        |stream| (stream, crate::http::Protocol::Http2),
-                    ))
-                    .finish(),
-            )
-            .timeout(Seconds(30))
-            .finish();
-
-        let url = format!("https://localhost:{}/", srv.addr.port());
-        let response = client.get(url).send().await.unwrap();
-        assert_eq!(response.version(), Version::HTTP_2);
-        assert!(response.status().is_success());
-    }
-
     #[cfg(feature = "cookie")]
     #[test]
     fn test_response_cookies() {
