@@ -1,23 +1,18 @@
-use std::{
-    error::Error as StdError, marker::PhantomData, pin::Pin, task::Context, task::Poll,
-};
-
-use bytes::Bytes;
-use futures_core::Stream;
-use futures_sink::Sink;
+use std::{error, marker::PhantomData, pin::Pin, task::Context, task::Poll};
 
 pub use crate::ws::{CloseCode, CloseReason, Frame, Message};
 
 use crate::http::body::{Body, BoxedBodyStream};
 use crate::http::error::PayloadError;
-use crate::http::ws::{handshake, HandshakeError};
 use crate::service::{IntoServiceFactory, Service, ServiceFactory};
 use crate::web::{HttpRequest, HttpResponse};
-use crate::{channel::mpsc, rt, ws};
+use crate::ws::{error::HandshakeError, handshake};
+use crate::{channel::mpsc, rt, util::Bytes, ws, Sink, Stream};
 
 pub type WebSocketsSink =
-    ws::StreamEncoder<mpsc::Sender<Result<Bytes, Box<dyn StdError>>>>;
+    ws::StreamEncoder<mpsc::Sender<Result<Bytes, Box<dyn error::Error>>>>;
 
+// TODO: fix close frame handling
 /// Do websocket handshake and start websockets service.
 pub async fn start<T, F, S, Err>(
     req: HttpRequest,
@@ -25,18 +20,11 @@ pub async fn start<T, F, S, Err>(
     factory: F,
 ) -> Result<HttpResponse, Err>
 where
-    T: ServiceFactory<
-        Config = WebSocketsSink,
-        Request = Frame,
-        Response = Option<Message>,
-    >,
-    T::Error: StdError + 'static,
-    T::InitError: 'static,
-    T::Service: 'static,
-    F: IntoServiceFactory<T>,
+    T: ServiceFactory<Frame, WebSocketsSink, Response = Option<Message>> + 'static,
+    T::Error: error::Error,
+    F: IntoServiceFactory<T, Frame, WebSocketsSink>,
     S: Stream<Item = Result<Bytes, PayloadError>> + Unpin + 'static,
-    Err: From<T::InitError>,
-    Err: From<HandshakeError>,
+    Err: From<T::InitError> + From<HandshakeError>,
 {
     let (tx, rx) = mpsc::channel();
 
@@ -52,21 +40,14 @@ pub async fn start_with<T, F, S, Err, Tx, Rx>(
     factory: F,
 ) -> Result<HttpResponse, Err>
 where
-    T: ServiceFactory<
-        Config = ws::StreamEncoder<Tx>,
-        Request = Frame,
-        Response = Option<Message>,
-    >,
-    T::Error: StdError + 'static,
-    T::InitError: 'static,
-    T::Service: 'static,
-    F: IntoServiceFactory<T>,
+    T: ServiceFactory<Frame, ws::StreamEncoder<Tx>, Response = Option<Message>> + 'static,
+    T::Error: error::Error,
+    F: IntoServiceFactory<T, Frame, ws::StreamEncoder<Tx>>,
     S: Stream<Item = Result<Bytes, PayloadError>> + Unpin + 'static,
-    Err: From<T::InitError>,
-    Err: From<HandshakeError>,
-    Tx: Sink<Result<Bytes, Box<dyn StdError>>> + Clone + Unpin + 'static,
-    Tx::Error: StdError,
-    Rx: Stream<Item = Result<Bytes, Box<dyn StdError>>> + Unpin + 'static,
+    Err: From<T::InitError> + From<HandshakeError>,
+    Tx: Sink<Result<Bytes, Box<dyn error::Error>>> + Clone + Unpin + 'static,
+    Tx::Error: error::Error,
+    Rx: Stream<Item = Result<Bytes, Box<dyn error::Error>>> + Unpin + 'static,
 {
     // ws handshake
     let mut res = handshake(req.head())?;
@@ -80,7 +61,7 @@ where
         .new_service(sink.clone())
         .await?
         .map_err(|e| {
-            let e: Box<dyn StdError> = Box::new(e);
+            let e: Box<dyn error::Error> = Box::new(e);
             e
         });
 
@@ -111,14 +92,11 @@ pin_project_lite::pin_project! {
 impl<S, I, E> Stream for MapStream<S, I, E>
 where
     S: Stream<Item = Result<I, E>>,
-    E: StdError + 'static,
+    E: error::Error + 'static,
 {
-    type Item = Result<I, Box<dyn StdError>>;
+    type Item = Result<I, Box<dyn error::Error>>;
 
-    fn poll_next(
-        self: Pin<&mut Self>,
-        cx: &mut Context<'_>,
-    ) -> Poll<Option<Self::Item>> {
+    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
         match self.project().stream.poll_next(cx) {
             Poll::Pending => Poll::Pending,
             Poll::Ready(Some(Ok(item))) => Poll::Ready(Some(Ok(item))),
